'use strict';

var Disaggregation = require('./Disaggregation'),
  Collection = require('../mvc/Collection'),
  Model = require('../mvc/Model'),
  Util = require('../util/Util');

// Default values to be used by constructor
var _DEFAULTS = {
  metadata: {
    rlabel: 'Distance',
    mlabel: 'Magnitude',
    εlabel: 'Contribution',
    εbins: [],
  },
  disaggs: [],
};

var _DEAGG_ID = 0;

/**
 * Class: DeaggResponse
 *
 * @param params {Object}
 *      Configuration options. See _DEFAULTS for more details.
 */
var DisaggResponse = function (params) {
  var _this, _initialize;

  // Inherit from parent class
  _this = Model();

  /**
   * @constructor
   *
   */
  _initialize = function (params) {
    var attributes, metadata, imt;

    params = Util.extend({}, _DEFAULTS, { id: 'disagg-response-' + _DEAGG_ID++ }, params);

    metadata = params.metadata;
    var disaggs = [];
    params.disaggs.forEach(function (disagg) {
      imt = disagg.imt;
      disagg.data.forEach(function (data) {
        var disagg = Disaggregation(
          Util.extend(
            {
              metadata,
            },
            data
          )
        );
        disaggs.push(disagg);
      });
    });

    attributes = {
      imt: imt.value,
      rlabel: metadata.rlabel,
      mlabel: metadata.mlabel,
      εlabel: metadata.εlabel,
      εbins: metadata.εbins,
      disaggregations: Collection(disaggs),
    };

    // Should not have listeners yet, but silent anyway to short-circuit check
    _this.set(attributes, { silent: true });
  };

  _this.destroy = Util.compose(function () {
    var disaggs;

    disaggs = _this.get('disaggregations');
    disaggs.destroy();

    _initialize = null;
    _this = null;
  }, _this.destroy);

  _initialize(params);
  params = null;
  return _this;
};

module.exports = DisaggResponse;
