export class Util {
  /**
   * Merge properties from a series of objects.
   *
   * @param dst
   *      target where merged properties are copied to.
   * @param <variable>
   *      source objects for properties. When a source is non null, it's
   *      properties are copied to the dst object. Properties are copied in
   *      the order of arguments: a property on a later argument overrides a
   *      property on an earlier argument.
   */
  static extend(dst: object): object;

  // remove an elements child nodes
  static empty(el: HTMLElement): void;

  /**
   * Creates a function that is a composition of other functions.
   *
   * For example:
   *      a(b(c(x))) === compose(c, b, a)(x);
   *
   * Each function should accept as an argument, the result of the previous
   * function call in the chain. It is allowable for all functions to have no
   * return value as well.
   *
   * @param functions variable set of functions to call, in order.
   *
   * @return The composition of the functions provided as arguments.
   */
  static compose(...functions: (() => {})[]): () => {};
}
